@@ -13,29 +13,26 @@
 // limitations under the License.
 
 //! OpenStack Identity V3 API support for access tokens.
-//!
-//! Start with creating an [Identity](struct.Identity.html) object which will
-//! guide you through setting all necessary values.
-//! [PasswordAuth](struct.PasswordAuth.html) is the actual
-//! implementation of the authentication [method](../trait.Method.html) trait.
-//!
-//! Note that as of now, only project-scoped tokens are supported.
-//! An attempt to create unscoped tokens always fails. This restriction may
-//! be lifted in the future.
 
+use std::collections::hash_map::DefaultHasher;
 use std::env;
+use std::fmt;
+use std::hash::{Hash, Hasher};
 use std::io::Read;
 
-use hyper::{Client, Url};
+use hyper::{Client, Get, Url};
 use hyper::Error as HttpClientError;
 use hyper::client::{IntoUrl, Response};
 use hyper::error::ParseError;
-use hyper::header::ContentType;
+use hyper::header::{ContentType, Headers};
+use hyper::method::Method;
 use hyper::status::StatusCode;
 
-use super::super::{ApiError, ApiResult, Session};
+use super::super::{ApiError, ApiResult};
 use super::super::identity::{catalog, protocol};
-use super::{Method, SimpleToken};
+use super::super::service::RequestBuilder;
+use super::super::utils::ValueCache;
+use super::AuthMethod;
 
 use ApiError::InvalidInput;
 
@@ -46,6 +43,25 @@ const MISSING_ENV_VARS: &'static str =
     "Not all required environment variables were provided";
 
 
+/// Plain authentication token without additional details.
+#[derive(Clone)]
+struct Token(pub String);
+
+impl fmt::Debug for Token {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let mut hasher = DefaultHasher::new();
+        self.hash(&mut hasher);
+        write!(f, "Token {{ hash: {} }}", hasher.finish())
+    }
+}
+
+impl Hash for Token {
+    fn hash<H>(&self, state: &mut H) where H: Hasher {
+        self.0.hash(state);
+    }
+}
+
+
 /// Authentication method factory using Identity API V3.
 #[derive(Clone, Debug)]
 pub struct Identity {
@@ -61,7 +77,8 @@ pub struct Identity {
 pub struct PasswordAuth {
     auth_url: Url,
     body: protocol::ProjectScopedAuthRoot,
-    token_endpoint: String
+    token_endpoint: String,
+    cached_token: ValueCache<Token>
 }
 
 impl Identity {
@@ -76,7 +93,7 @@ impl Identity {
         Ok(Identity {
             auth_url: real_url,
             password_identity: None,
-            project_scope: None
+            project_scope: None,
         })
     }
 
@@ -170,12 +187,13 @@ impl PasswordAuth {
         PasswordAuth {
             auth_url: auth_url,
             body: body,
-            token_endpoint: token_endpoint
+            token_endpoint: token_endpoint,
+            cached_token: ValueCache::new(None)
         }
     }
 
     fn token_from_response(&self, mut resp: Response)
-            -> ApiResult<SimpleToken> {
+            -> ApiResult<Token> {
         let mut resp_body = String::new();
         let _ignored = try!(resp.read_to_string(&mut resp_body));
 
@@ -211,37 +229,58 @@ impl PasswordAuth {
 
         // TODO: detect expiration time
         // TODO: do something useful about the body
-        Ok(SimpleToken(token_value))
+        Ok(Token(token_value))
+    }
+
+    fn refresh_token(&self, client: &Client) -> ApiResult<()> {
+        // TODO: refresh on expiration
+        self.cached_token.ensure_value(|| {
+            debug!("Requesting a token for user {} from {}",
+                   self.body.auth.identity.password.user.name,
+                   self.token_endpoint);
+            let body = self.body.to_string().unwrap();
+            let resp = try!(client.post(&self.token_endpoint).body(&body)
+                            .header(ContentType::json()).send());
+            self.token_from_response(resp)
+        })
+    }
+
+    fn get_token(&self, client: &Client) -> ApiResult<String> {
+        try!(self.refresh_token(client));
+        Ok(self.cached_token.get().unwrap().0)
+    }
+
+    fn get_catalog(&self, client: &Client)
+            -> ApiResult<Vec<protocol::CatalogRecord>> {
+        // TODO: catalog caching
+        let catalog_url = catalog::get_url(self.auth_url.clone());
+        trace!("Requesting a service catalog from {}", catalog_url);
+        let req = try!(self.request(client, Get, catalog_url, Headers::new()));
+        let body: protocol::CatalogRoot = try!(req.fetch_json());
+        trace!("Received catalog: {:?}", body.catalog);
+        Ok(body.catalog)
     }
 }
 
-impl Method for PasswordAuth {
-    // TODO: implement its own token type
-    type TokenType = SimpleToken;
-
-    /// Verify authentication and generate an auth token.
-    fn get_token(&self, client: &Client) -> ApiResult<SimpleToken> {
-        debug!("Requesting a token for user {} from {}",
-               self.body.auth.identity.password.user.name,
-               self.token_endpoint);
-
-        let body = self.body.to_string().unwrap();
-        let resp = try!(client.post(&self.token_endpoint).body(&body)
-                        .header(ContentType::json()).send());
-        self.token_from_response(resp)
+impl AuthMethod for PasswordAuth {
+    /// Create an authenticated request.
+    fn request<'a>(&self, client: &'a Client, method: Method, url: Url,
+                   headers: Headers) -> ApiResult<RequestBuilder<'a>> {
+        let token = try!(self.get_token(client));
+        Ok(RequestBuilder::new(client, method, url, headers)
+           .header(protocol::AuthTokenHeader(token)))
     }
 
     /// Get a URL for the requested service.
-    fn get_endpoint(&self, service_type: String,
+    fn get_endpoint(&self, client: &Client,
+                    service_type: String,
                     endpoint_interface: Option<String>,
-                    region: Option<String>,
-                    client: &Session<PasswordAuth>)
-            -> ApiResult<Url> {
+                    region: Option<String>) -> ApiResult<Url> {
         let real_interface = endpoint_interface.unwrap_or(
             self.default_endpoint_interface());
         debug!("Requesting a catalog endpoint for service '{}', interface \
                '{}' from region {:?}", service_type, real_interface, region);
-        let cat = try!(catalog::get_service_catalog(&self.auth_url, client));
+        let cat = try!(self.get_catalog(client));
         let endp = try!(catalog::find_endpoint(&cat, service_type,
                                                real_interface, region));
         info!("Received {:?}", endp);
@@ -257,10 +296,9 @@ pub mod test {
     use hyper::{self, Url};
     use hyper::status::StatusCode;
 
-    use super::super::super::{ApiError, ApiResult, Session};
-    use super::super::{Method, SimpleToken};
-    use super::{Identity, PasswordAuth};
-    use super::super::super::session::test;
+    use super::super::super::{ApiError, ApiResult};
+    use super::super::AuthMethod;
+    use super::Identity;
 
     mock_connector!(MockToken {
         "http://127.0.1.1" => "HTTP/1.1 200 OK\r\n\
@@ -380,7 +418,7 @@ pub mod test {
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockToken::default());
         let token = id.get_token(&cli).unwrap();
-        assert_eq!(&token.to_string(), "abcdef");
+        assert_eq!(&token, "abcdef");
     }
 
     #[test]
@@ -409,44 +447,32 @@ pub mod test {
         };
     }
 
-    fn new_session() -> Session<PasswordAuth> {
+    fn get_endpoint(service_type: &str, interface_endpoint: Option<&str>,
+                    region: Option<&str>) -> ApiResult<Url> {
         let id = Identity::new("http://127.0.2.1").unwrap()
             .with_user("user", "pa$$w0rd", "example.com")
             .with_project_scope("cool project", "example.com")
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockCatalog::default());
-        let token = SimpleToken(String::from("abcdef"));
-        test::new_with_params(id, cli, token, None)
-    }
-
-    fn get_endpoint(session: &Session<PasswordAuth>, service_type: &str,
-                    interface_endpoint: Option<&str>, region: Option<&str>)
-            -> ApiResult<Url> {
-        session.auth_method().get_endpoint(
-            String::from(service_type),
-            interface_endpoint.map(String::from),
-            region.map(String::from),
-            session)
+        id.get_endpoint(&cli, String::from(service_type),
+                        interface_endpoint.map(String::from),
+                        region.map(String::from))
     }
 
     #[test]
     fn test_identity_get_endpoint() {
-        let session = new_session();
-
-        let e1 = get_endpoint(&session, "identity", None, None).unwrap();
+        let e1 = get_endpoint("identity", None, None).unwrap();
         assert_eq!(&e1.to_string(), "http://localhost:5000/");
-        let e2 = get_endpoint(&session, "identity", Some("admin"), None)
-            .unwrap();
+        let e2 = get_endpoint("identity", Some("admin"), None).unwrap();
         assert_eq!(&e2.to_string(), "http://localhost:35357/");
 
-        match get_endpoint(&session, "foo", None, None).err().unwrap() {
+        match get_endpoint("foo", None, None).err().unwrap() {
             ApiError::EndpointNotFound(ref endp) =>
                 assert_eq!(endp, "foo"),
             other => panic!("Unexpected {}", other)
         };
 
-        match get_endpoint(&session, "identity", Some("unknown"), None)
-                .err().unwrap() {
+        match get_endpoint("identity", Some("unknown"), None).err().unwrap() {
             ApiError::EndpointNotFound(ref endp) =>
                 assert_eq!(endp, "identity"),
             other => panic!("Unexpected {}", other)
@@ -455,13 +481,11 @@ pub mod test {
 
     #[test]
     fn test_identity_get_endpoint_with_region() {
-        let session = new_session();
-
-        let e1 = get_endpoint(&session, "identity", Some("admin"),
+        let e1 = get_endpoint("identity", Some("admin"),
                               Some("RegionOne")).unwrap();
         assert_eq!(&e1.to_string(), "http://localhost:35357/");
 
-        match get_endpoint(&session, "identity", None,
+        match get_endpoint("identity", None,
                            Some("unknown")).err().unwrap() {
             ApiError::EndpointNotFound(ref endp) =>
                 assert_eq!(endp, "identity"),
