@@ -17,15 +17,20 @@
 #![allow(non_snake_case)]
 #![allow(missing_docs)]
 
+use std::collections::HashMap;
+use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
 use std::str::FromStr;
 
+use chrono::{DateTime, FixedOffset};
 use hyper::Url;
+use serde::{Deserialize, Deserializer};
 use serde::de::Error as DeserError;
 use serde_json::Error as JsonError;
 
 use super::super::super::{ApiResult, ApiVersion};
 use super::super::super::ApiError::InvalidJson;
 use super::super::super::service::ServiceInfo;
+use super::super::super::utils;
 
 
 /// Available sort keys.
@@ -64,39 +69,104 @@ pub enum ServerSortKey {
     __Nonexhaustive,
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct Server {
-    pub accessIPv4: String,
-    pub accessIPv6: String,
+/// All possible server statuses.
+#[derive(Debug, Copy, Clone)]
+pub enum ServerStatus {
+    Active,
+    Building,
+    Deleted,
+    Error,
+    HardRebooting,
+    Migrating,
+    Paused,
+    Rebooting,
+    Resizing,
+    RevertingResize,
+    ShutOff,
+    Suspended,
+    Rescuing,
+    Shelved,
+    ShelvedOffloaded,
+    SoftDeleted,
+    Unknown,
+    UpdatingPassword,
+    VerifyingResize,
+    #[doc(hidden)]
+    __Nonexhaustive,
+}
+
+/// Address of a server.
+#[derive(Clone, Copy, Debug)]
+pub enum AddressType {
+    Fixed,
+    Floating,
+    Unknown
+}
+
+/// Address of a server.
+#[derive(Clone, Debug, Deserialize)]
+pub struct ServerAddress {
+    /// IP (v4 of v6) address.
+    pub addr: IpAddr,
+    /// MAC address (if available).
+    #[serde(rename = "OS-EXT-IPS-MAC:mac_addr")]
+    pub mac_addr: Option<String>,
+    /// Address type (if known).
+    #[serde(rename = "OS-EXT-IPS:type", default,
+            deserialize_with = "de_address_type")]
+    pub addr_type: AddressType
+}
+
+#[derive(Clone, Debug, Deserialize)]
+pub struct Ref {
     pub id: String,
+    pub links: Vec<Link>
+}
+
+#[derive(Clone, Debug, Deserialize)]
+pub struct Server {
+    #[serde(deserialize_with = "utils::empty_as_none")]
+    pub accessIPv4: Option<Ipv4Addr>,
+    #[serde(deserialize_with = "utils::empty_as_none")]
+    pub accessIPv6: Option<Ipv6Addr>,
+    #[serde(default)]
+    pub addresses: HashMap<String, Vec<ServerAddress>>,
+    #[serde(rename = "OS-EXT-AZ:availability_zone")]
+    pub availability_zone: String,
+    pub created: DateTime<FixedOffset>,
+    pub flavor: Ref,
+    pub id: String,
+    pub image: Ref,
     pub name: String,
-    pub status: String,
+    #[serde(deserialize_with = "de_server_status", default)]
+    pub status: ServerStatus,
     pub tenant_id: String,
+    pub updated: DateTime<FixedOffset>,
     pub user_id: String
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct ServerSummary {
     pub id: String,
     pub name: String,
 }
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct ServersRoot {
     pub servers: Vec<ServerSummary>
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct ServerRoot {
     pub server: Server
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct Link {
     pub href: String,
     pub rel: String
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct Version {
     pub id: String,
     pub links: Vec<Link>,
@@ -105,12 +175,12 @@ pub struct Version {
     pub min_version: String
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct VersionsRoot {
     pub versions: Vec<Version>
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Deserialize)]
 pub struct VersionRoot {
     pub version: Version
 }
@@ -185,3 +255,54 @@ impl Into<String> for ServerSortKey {
         })
     }
 }
+
+impl Default for ServerStatus {
+    fn default() -> ServerStatus {
+        ServerStatus::Unknown
+    }
+}
+
+impl Default for AddressType {
+    fn default() -> AddressType {
+        AddressType::Unknown
+    }
+}
+
+fn de_address_type<D>(des: D) -> Result<AddressType, D::Error>
+        where D: Deserializer {
+    let s = try!(String::deserialize(des));
+    Ok(match s.as_ref() {
+        "fixed" => AddressType::Fixed,
+        "floating" => AddressType::Floating,
+        _ => Default::default()
+    })
+}
+
+fn de_server_status<D>(des: D) -> Result<ServerStatus, D::Error>
+        where D: Deserializer {
+    let s = try!(String::deserialize(des));
+    Ok(match s.as_ref() {
+        "ACTIVE" => ServerStatus::Active,
+        "BUILD" => ServerStatus::Building,
+        "DELETED" => ServerStatus::Deleted,
+        "ERROR" => ServerStatus::Error,
+        "HARD_REBOOT" => ServerStatus::HardRebooting,
+        "MIGRATING" => ServerStatus::Migrating,
+        "PAUSED" => ServerStatus::Paused,
+        "REBOOT" => ServerStatus::Rebooting,
+        "RESIZE" => ServerStatus::Resizing,
+        "REVERT_RESIZE" => ServerStatus::RevertingResize,
+        "SHUTOFF" => ServerStatus::ShutOff,
+        "SUSPENDED" => ServerStatus::Suspended,
+        "RESCUE" => ServerStatus::Rescuing,
+        "SHELVED" => ServerStatus::Shelved,
+        "SHELVED_OFFLOADED" => ServerStatus::ShelvedOffloaded,
+        "SOFT_DELETED" => ServerStatus::SoftDeleted,
+        "PASSWORD" => ServerStatus::UpdatingPassword,
+        "VERIFY_RESIZE" => ServerStatus::VerifyingResize,
+        _ => {
+            warn!("Got unknown server status {}", s);
+            Default::default()
+        }
+    })
+}
