@@ -16,25 +16,23 @@
 
 use std::marker::PhantomData;
 
-use hyper::Url;
-use hyper::client::{Body, RequestBuilder as HyperRequestBuilder, Response};
+use hyper::{Client, Url};
+use hyper::client::{Body, IntoUrl, RequestBuilder as HyperRequestBuilder,
+                    Response};
 use hyper::header::{Header, HeaderFormat, Headers};
 pub use hyper::method::Method;
 use serde::{Deserialize, Serialize};
 use serde_json;
 
 use super::{ApiError, ApiResult, ApiVersion, ApiVersionRequest, Session};
-use super::auth::Method as AuthMethod;
-use super::identity::protocol::AuthTokenHeader;
 use super::utils;
 
 
-/// Request builder with authentication, error checking and JSON support.
+/// Request builder with error checking and JSON support.
 ///
 /// Partly copies the interface of hyper::client::RequestBuilder.
 #[allow(missing_debug_implementations)]
-pub struct RequestBuilder<'a, A: AuthMethod + 'a> {
-    parent: &'a Session<A>,
+pub struct RequestBuilder<'a> {
     inner: HyperRequestBuilder<'a>
 }
 
@@ -59,7 +57,7 @@ pub trait ServiceType {
     fn catalog_type() -> &'static str;
 
     /// Get basic service information.
-    fn service_info<Auth: AuthMethod>(endpoint: Url, session: &Session<Auth>)
+    fn service_info(endpoint: Url, session: &Session)
         -> ApiResult<ServiceInfo>;
 }
 
@@ -71,9 +69,8 @@ pub trait ApiVersioning {
 
 /// A service-specific wrapper around Session.
 #[derive(Debug)]
-pub struct ServiceWrapper<'session, Auth: AuthMethod + 'session,
-                          Srv: ServiceType> {
-    session: &'session Session<Auth>,
+pub struct ServiceWrapper<'session, Srv: ServiceType> {
+    session: &'session Session,
     service_type: PhantomData<Srv>,
     endpoint_interface: Option<String>
 }
@@ -98,13 +95,12 @@ impl Query {
     }
 }
 
-impl<'a, Auth: AuthMethod> RequestBuilder<'a, Auth> {
+impl<'a> RequestBuilder<'a> {
     /// Wrap a request builder.
-    pub fn new(inner: HyperRequestBuilder<'a>, session: &'a Session<Auth>)
-            -> RequestBuilder<'a, Auth> {
+    pub fn new<U>(client: &'a Client, method: Method, url: U, headers: Headers)
+            -> RequestBuilder<'a> where U: IntoUrl {
         RequestBuilder {
-            parent: session,
-            inner: inner
+            inner: client.request(method, url).headers(headers)
         }
     }
 
@@ -120,9 +116,7 @@ impl<'a, Auth: AuthMethod> RequestBuilder<'a, Auth> {
 
     /// Send this request without checking on status code.
     pub fn send_unchecked(self) -> ApiResult<Response> {
-        let token = try!(self.parent.auth_token());
-        let hdr = AuthTokenHeader(token.into());
-        self.inner.header(hdr).send().map_err(From::from).map(|resp| {
+        self.inner.send().map_err(From::from).map(|resp| {
             trace!("Got {} from {} with {:?}",
                    resp.status, resp.url, resp.headers);
             resp
@@ -136,40 +130,24 @@ impl<'a, Auth: AuthMethod> RequestBuilder<'a, Auth> {
     }
 
     /// Add body to the request.
-    pub fn body<B: Into<Body<'a>>>(self, body: B)
-            -> RequestBuilder<'a, Auth> {
+    pub fn body<B: Into<Body<'a>>>(self, body: B) -> RequestBuilder<'a> {
         RequestBuilder {
-            inner: self.inner.body(body),
-            .. self
-        }
-    }
-
-    /// Add additional headers to the request.
-    pub fn headers(self, headers: Headers)
-            -> RequestBuilder<'a, Auth> {
-        RequestBuilder {
-            inner: self.inner.headers(headers),
-            .. self
+            inner: self.inner.body(body)
         }
     }
 
     /// Add an individual header to the request.
-    ///
-    /// Note that X-Auth-Token is always overwritten with a token in use.
     pub fn header<H: Header + HeaderFormat>(self, header: H)
-            -> RequestBuilder<'a, Auth> {
+            -> RequestBuilder<'a> {
         RequestBuilder {
-            inner: self.inner.header(header),
-            .. self
+            inner: self.inner.header(header)
         }
     }
 }
 
-impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
-        ServiceWrapper<'session, Auth, Srv> {
+impl<'session, Srv: ServiceType> ServiceWrapper<'session, Srv> {
     /// Create a new wrapper for the specific service.
-    pub fn new(session: &'session Session<Auth>)
-            -> ServiceWrapper<'session, Auth, Srv> {
+    pub fn new(session: &'session Session) -> ServiceWrapper<'session, Srv> {
         ServiceWrapper {
             session: session,
             service_type: PhantomData,
@@ -179,7 +157,7 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
 
     /// Change the endpoint interface used for this wrapper.
     pub fn with_endpoint_interface(self, endpoint_interface: String)
-            -> ServiceWrapper<'session, Auth, Srv> {
+            -> ServiceWrapper<'session, Srv> {
         ServiceWrapper {
             endpoint_interface: Some(endpoint_interface),
             .. self
@@ -198,13 +176,13 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
 
     /// Make an HTTP request to the given service.
     pub fn request<P>(&'session self, method: Method, path: P, query: Query)
-            -> ApiResult<RequestBuilder<'session, Auth>>
+            -> ApiResult<RequestBuilder<'session>>
             where P: IntoIterator, P::Item: AsRef<str> {
         let url = try!(self.get_endpoint(path, query));
         let headers = self.session.service_headers::<Srv>();
         trace!("Sending HTTP {} request to {} with {:?}",
                method, url, headers);
-        Ok(self.session.raw_request(method, url).headers(headers))
+        self.session.request(method, url, headers)
     }
 
     /// Make an HTTP request with JSON body and JSON response.
@@ -251,9 +229,8 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
     }
 }
 
-impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
-        Clone for ServiceWrapper<'session, Auth, Srv> {
-    fn clone(&self) -> ServiceWrapper<'session, Auth, Srv> {
+impl<'session, Srv: ServiceType> Clone for ServiceWrapper<'session, Srv> {
+    fn clone(&self) -> ServiceWrapper<'session, Srv> {
         ServiceWrapper {
             session: self.session,
             service_type: PhantomData,
