@@ -17,13 +17,13 @@
 use std::cell::Ref;
 use std::collections::HashMap;
 
-use hyper::{Client, Url};
-use hyper::client::IntoUrl;
+use hyper::{Client, Get, Url};
+use hyper::client::{IntoUrl, Response};
 use hyper::header::Headers;
 use hyper::method::Method;
 
 use super::{ApiError, ApiResult, ApiVersion, ApiVersionRequest};
-use super::auth::Method as AuthMethod;
+use super::auth::AuthMethod;
 use super::service::{ApiVersioning, RequestBuilder, ServiceInfo, ServiceType};
 use super::utils;
 
@@ -37,10 +37,9 @@ use super::utils;
 ///
 /// Finally, the session object is responsible for API version negotiation.
 #[derive(Debug)]
-pub struct Session<Auth: AuthMethod> {
-    auth: Auth,
+pub struct Session {
+    auth: Box<AuthMethod>,
     client: Client,
-    cached_token: utils::ValueCache<Auth::TokenType>,
     cached_info: utils::MapCache<(&'static str, String), ServiceInfo>,
     api_versions: HashMap<&'static str, (ApiVersion, Headers)>,
     region: Option<String>,
@@ -48,18 +47,17 @@ pub struct Session<Auth: AuthMethod> {
 }
 
 
-impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
+impl Session {
     /// Create a new session with a given authentication plugin.
     ///
     /// The resulting session will use the default endpoint interface (usually,
     /// public) and the first available region.
-    pub fn new(auth_method: Auth) -> Session<Auth> {
+    pub fn new<Auth: AuthMethod + 'static>(auth_method: Auth) -> Session {
         let ep = auth_method.default_endpoint_interface();
         let region = auth_method.default_region();
         Session {
-            auth: auth_method,
+            auth: Box::new(auth_method),
             client: utils::http_client(),
-            cached_token: utils::ValueCache::new(None),
             cached_info: utils::MapCache::new(),
             api_versions: HashMap::new(),
             region: region,
@@ -70,11 +68,10 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
     /// Convert this session into one using the given region.
     ///
     /// Negotiated API versions are reset to their default values.
-    pub fn with_region<S: Into<String>>(self, region: S) -> Session<Auth> {
+    pub fn with_region<S: Into<String>>(self, region: S) -> Session {
         Session {
             auth: self.auth,
             client: self.client,
-            cached_token: self.cached_token,
             // ServiceInfo has to be refreshed
             cached_info: utils::MapCache::new(),
             // Different regions potentially have different API versions?
@@ -87,13 +84,11 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
     /// Convert this session into one using the given endpoint interface.
     ///
     /// Negotiated API versions are kept in the new object.
-    pub fn with_endpoint_interface<S: Into<String>>(self,
-                                                    endpoint_interface: S)
-            -> Session<Auth> {
+    pub fn with_endpoint_interface<S>(self, endpoint_interface: S)
+            -> Session where S: Into<String> {
         Session {
             auth: self.auth,
             client: self.client,
-            cached_token: self.cached_token,
             // ServiceInfo has to be refreshed
             cached_info: utils::MapCache::new(),
             api_versions: self.api_versions,
@@ -102,15 +97,9 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
         }
     }
 
-    /// Get a clone of the authentication token.
-    pub fn auth_token(&self) -> ApiResult<Auth::TokenType> {
-        try!(self.refresh_token());
-        Ok(self.cached_token.get().unwrap())
-    }
-
     /// Get a reference to the authentication method in use.
-    pub fn auth_method(&self) -> &Auth {
-        &self.auth
+    pub fn auth_method(&self) -> &AuthMethod {
+        self.auth.as_ref()
     }
 
     /// Get an API version used for given service.
@@ -126,12 +115,6 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
             .unwrap_or_else(Headers::new)
     }
 
-    /// A wrapper for HTTP request.
-    pub fn raw_request<U: IntoUrl>(&'a self, method: Method, url: U)
-            -> RequestBuilder<'a, Auth> {
-        RequestBuilder::new(self.client.request(method, url), self)
-    }
-
     /// Get service info for the given service.
     ///
     /// If endpoint interface is not provided, the default for this session
@@ -179,10 +162,16 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
         }
     }
 
-    fn refresh_token(&self) -> ApiResult<()> {
-        self.cached_token.ensure_value(|| {
-            self.auth.get_token(&self.client)
-        })
+    /// Prepare an HTTP request with authentication.
+    pub fn request<'a, U>(&'a self, method: Method, url: U, headers: Headers)
+            -> ApiResult<RequestBuilder<'a>> where U: IntoUrl {
+        let real_url = try!(url.into_url());
+        self.auth.request(&self.client, method, real_url, headers)
+    }
+
+    /// Send a simple GET request.
+    pub fn http_get<U>(&self, url: U) -> ApiResult<Response> where U: IntoUrl {
+        try!(self.request(Get, url, Headers::new())).send()
     }
 
     fn ensure_service_info<Srv>(&self, endpoint_interface: String)
@@ -199,10 +188,10 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
 
     fn get_catalog_endpoint<S>(&self, service_type: S) -> ApiResult<Url>
             where S: Into<String> {
-        self.auth.get_endpoint(service_type.into(),
+        self.auth.get_endpoint(&self.client,
+                               service_type.into(),
                                Some(self.endpoint_interface.clone()),
-                               self.region.clone(),
-                               &self)
+                               self.region.clone())
     }
 
     fn get_service_info_ref<Srv>(&self, endpoint_interface: String)
@@ -213,13 +202,12 @@ impl<'a, Auth: AuthMethod + 'a> Session<Auth> {
 }
 
 
-impl<Auth: AuthMethod + Clone> Clone for Session<Auth> {
-    fn clone(&self) -> Session<Auth> {
+impl Clone for Session {
+    fn clone(&self) -> Session {
         Session {
             auth: self.auth.clone(),
             // NOTE: hyper::Client does not support Clone
             client: utils::http_client(),
-            cached_token: self.cached_token.clone(),
             cached_info: self.cached_info.clone(),
             api_versions: self.api_versions.clone(),
             region: self.region.clone(),
@@ -238,24 +226,21 @@ pub mod test {
     use std::io::Read;
 
     use hyper;
-    use hyper::header::ContentLength;
+    use hyper::header::{ContentLength, Headers};
     use hyper::status::StatusCode;
 
     use super::super::ApiError;
-    use super::super::auth::{Identity, Method, Token, NoAuth, SimpleToken};
-    use super::super::auth::identity::PasswordAuth;
+    use super::super::auth::{AuthMethod, Identity, NoAuth};
     use super::super::utils;
     use super::Session;
 
-    pub fn new_with_params<Auth: Method>(auth: Auth, cli: hyper::Client,
-                                         token: Auth::TokenType,
-                                         region: Option<&str>)
-            -> Session<Auth> {
+    pub fn new_with_params<Auth>(auth: Auth, cli: hyper::Client,
+                                 region: Option<&str>)
+            -> Session where Auth: AuthMethod + 'static {
         let ep = auth.default_endpoint_interface();
         Session {
-            auth: auth,
+            auth: Box::new(auth),
             client: cli,
-            cached_token: utils::ValueCache::new(Some(token)),
             cached_info: utils::MapCache::new(),
             api_versions: HashMap::new(),
             region: region.map(From::from),
@@ -263,24 +248,16 @@ pub mod test {
         }
     }
 
-    pub fn new_session(token: &str) -> Session<NoAuth> {
-        let token = SimpleToken(String::from(token));
-        new_with_params(NoAuth::new("http://127.0.0.1/").unwrap(),
-                        utils::http_client(), token, None)
-    }
-
-    fn session_with_identity() -> Session<PasswordAuth> {
+    fn session_with_identity() -> Session {
         let id = Identity::new("http://127.0.2.1").unwrap()
             .with_user("user", "pa$$w0rd", "example.com")
             .with_project_scope("cool project", "example.com")
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockCatalog::default());
-        let token = SimpleToken(String::from("abcdef"));
         let ep = id.default_endpoint_interface();
         Session {
-            auth: id,
+            auth: Box::new(id),
             client: cli,
-            cached_token: utils::ValueCache::new(Some(token)),
             cached_info: utils::MapCache::new(),
             api_versions: HashMap::new(),
             region: None,
@@ -344,20 +321,13 @@ pub mod test {
     });
 
     #[test]
-    fn test_session_new() {
-        let s = new_session("foo");
-        let token = s.auth_token().unwrap();
-        assert_eq!(&token.to_string(), "foo");
-        assert!(!token.needs_refresh());
-    }
-
-    #[test]
-    fn test_session_raw_request() {
+    fn test_session_request() {
         let cli = hyper::Client::with_connector(MockHttp::default());
         let s = new_with_params(NoAuth::new("http://127.0.0.1/").unwrap(),
-                                cli, SimpleToken(String::from("token")), None);
+                                cli, None);
 
-        let mut resp = s.raw_request(hyper::Post, "http://127.0.0.1/")
+        let mut resp = s.request(hyper::Post, "http://127.0.0.1/",
+                                 Headers::new()).unwrap()
             .body("body").header(ContentLength(4u64)).send().unwrap();
 
         let mut s = String::new();
@@ -366,12 +336,13 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_raw_request_error() {
+    fn test_session_request_error() {
         let cli = hyper::Client::with_connector(MockHttp::default());
         let s = new_with_params(NoAuth::new("http://127.0.0.2/").unwrap(),
-                                cli, SimpleToken(String::from("token")), None);
+                                cli, None);
 
-        let err = s.raw_request(hyper::Post, "http://127.0.0.2/")
+        let err = s.request(hyper::Post, "http://127.0.0.2/",
+                            Headers::new()).unwrap()
             .body("body").header(ContentLength(4u64)).send().err().unwrap();
 
         match err {
@@ -381,12 +352,13 @@ pub mod test {
     }
 
     #[test]
-    fn test_session_raw_request_unchecked_error() {
+    fn test_session_request_unchecked_error() {
         let cli = hyper::Client::with_connector(MockHttp::default());
         let s = new_with_params(NoAuth::new("http://127.0.0.2/").unwrap(),
-                                cli, SimpleToken(String::from("token")), None);
+                                cli, None);
 
-        let mut resp = s.raw_request(hyper::Post, "http://127.0.0.2/")
+        let mut resp = s.request(hyper::Post, "http://127.0.0.2/",
+                                 Headers::new()).unwrap()
             .body("body").header(ContentLength(4u64)).send_unchecked()
             .unwrap();
 
